(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUVÉ language                *)
(*                                                                        *)
(*  Copyright (C) 2004-2009 Ralf Treinen                                  *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@pps.jussieu.fr>                                 *)
(*  Laboratoire Preuves, Programmes et Systèmes (PPS),                   *)
(*  Université Paris Diderot, Paris, France                              *)
(*  http://www.pps.jussieu.fr/~treinen                                    *)
(*                                                                        *)
(*  This software was mainly written while the author was with the        *)
(*  Laboratoire Spécification et Vérification (LSV),                    *)
(*  ENS de Cachan, Cachan, France                                         *)
(**************************************************************************)

(**************************************************************************)
(* Equational Axioms                                                      *)
(**************************************************************************)

(* the parser guarantees that every expression in the second component *)
(* of an element of type "axiom" passes "is_axiom", that is that it is *)
(* an equation between two functional expressions.                     *)
type axioms = Declarations.declaration list * Expressions.expression list

(* (is_axiom a) checks that a is an axiom, that is an equality between *)
(* two functional expressions *)
val is_axiom: Expressions.expression -> bool

(* (is_a_axiom a) returns Some(f) if a is the associativity axiom of f,   *)
(* otherwise it returns None. Likewise for as_c_axiom.                    *)
val is_a_axiom: Expressions.expression -> string option
val is_c_axiom: Expressions.expression -> string option

(* check that axioms is well-formed:                                      *)
(* - the declaration list must be well-formed                             *)
(* - no declaration may be mutable                                        *)
(* - both sides of an equation have the same type                         *)
(* and annotate all expressions thereinf with type and kind               *)
exception Exc_axioms of string
val type_axioms: Signatures.simple_signature -> axioms -> axioms

