(**************************************************************************)
(*  ProuveParser: parser frontend for the PROUVÉ language                *)
(*                                                                        *)
(*  Copyright (C) 2004-2009 Ralf Treinen                                  *)
(*                                                                        *)
(*  This library is free software; you can redistribute it and/or         *)
(*  modify it under the terms of the GNU Lesser General Public            *)
(*  License as published by the Free Software Foundation; either          *)
(*  version 2.1 of the License, or (at your option) any later version.    *)
(*                                                                        *)
(*  This library is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *)
(*  Lesser General Public License for more details.                       *)
(*                                                                        *)
(*  You should have received a copy of the GNU Lesser General Public      *)
(*  License along with this library; if not, write to the Free Software   *)
(*  Foundation, Inc., 59 Temple Place, Suite 330,                         *)
(*  Boston, MA  02111-1307  USA                                           *)
(*                                                                        *)
(*  Ralf Treinen <treinen@pps.jussieu.fr>                                 *)
(*  Laboratoire Preuves, Programmes et Systèmes (PPS),                   *)
(*  Université Paris Diderot, Paris, France                              *)
(*  http://www.pps.jussieu.fr/~treinen                                    *)
(*                                                                        *)
(*  This software was mainly written while the author was with the        *)
(*  Laboratoire Spécification et Vérification (LSV),                    *)
(*  ENS de Cachan, Cachan, France                                         *)
(**************************************************************************)

(* Instantiation analysis of expressions and patterns *)

open Expressions

(* (is_instantiated iv exp), where iv is a list                         *)
(* of instantiated variables, returns expression exp with the has_value *)
(* fields of all identifiers turned to true in case all identifiers of  *)
(* exp are in iv, otherwise raises exception Noninst.                   *)
exception Noninst of string
val is_instantiated : string list -> expression -> expression

(* (is_inst_pattern hf iv pat), where iv is a list of instantiated variables,*)
(* and hf is the list of hash functions,                                   *)
(* checks for every expression e in pattern pat whether all non-yet        *)
(* instantiated variables of e can be instantiated by a successful pattern *)
(* matching. If yes it it checks that a succesful pattern matching by all  *)
(* expression e in pat yield the same set of instantiated variables.       *)
(* If both tests succeed then the function returns the same pattern with   *)
(* the has_value updated and the list of variables instantiated after      *)
(* pattern matching,  otherwise it raises the exception Noninst.           *)
val is_inst_pattern : string list -> string list ->
  pattern -> (pattern * string list)
